package com.ruoyi.druid.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.druid.mapper.DruidConfigMapper;
import com.ruoyi.druid.domain.DruidConfig;
import com.ruoyi.druid.service.IDruidConfigService;

/**
 * 多数据源配置Service业务层处理
 *
 * @author ruoyi
 * @date 2024-07-14
 */
@Service
public class DruidConfigServiceImpl implements IDruidConfigService
{
    @Autowired
    private DruidConfigMapper druidConfigMapper;

    /**
     * 查询多数据源配置
     *
     * @param id 多数据源配置主键
     * @return 多数据源配置
     */
    @Override
    public DruidConfig selectDruidConfigById(Long id)
    {
        return druidConfigMapper.selectDruidConfigById(id);
    }

    /**
     * 查询多数据源配置列表
     *
     * @param tbDruidConfig 多数据源配置
     * @return 多数据源配置
     */
    @Override
    public List<DruidConfig> selectDruidConfigList(DruidConfig tbDruidConfig)
    {
        return druidConfigMapper.selectDruidConfigList(tbDruidConfig);
    }

    /**
     * 新增多数据源配置
     *
     * @param tbDruidConfig 多数据源配置
     * @return 结果
     */
    @Override
    public int insertDruidConfig(DruidConfig tbDruidConfig)
    {
        tbDruidConfig.setCreateTime(DateUtils.getNowDate());
        return druidConfigMapper.insertDruidConfig(tbDruidConfig);
    }

    /**
     * 修改多数据源配置
     *
     * @param tbDruidConfig 多数据源配置
     * @return 结果
     */
    @Override
    public int updateDruidConfig(DruidConfig tbDruidConfig)
    {
        tbDruidConfig.setUpdateTime(DateUtils.getNowDate());
        return druidConfigMapper.updateDruidConfig(tbDruidConfig);
    }

    /**
     * 批量删除多数据源配置
     *
     * @param ids 需要删除的多数据源配置主键
     * @return 结果
     */
    @Override
    public int deleteDruidConfigByIds(Long[] ids)
    {
        return druidConfigMapper.deleteDruidConfigByIds(ids);
    }

    /**
     * 删除多数据源配置信息
     *
     * @param id 多数据源配置主键
     * @return 结果
     */
    @Override
    public int deleteDruidConfigById(Long id)
    {
        return druidConfigMapper.deleteDruidConfigById(id);
    }
}
